/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file DWaitStateCollReduction.h
 *       @see DWaitStateCollReduction.
 *
 *  @date 05.03.2013
 *  @author Tobias Hilbrich
 */

#include "ModuleBase.h"
#include "I_ParallelIdAnalysis.h"
#include "I_CommTrack.h"
#include "I_DWaitStateCollReduction.h"
#include "I_CollCommListener.h"
#include "I_DCollectiveMatchReduction.h"
#include "DistributedDeadlockApi.h"

#include <list>

#ifndef DWAITSTATECOLLREDUCTION_H
#define DWAITSTATECOLLREDUCTION_H

using namespace gti;

namespace must
{
    /**
     * Implementation of I_DWaitStateCollReduction.
     * @see I_DWaitStateCollReduction
     */
    class DWaitStateCollReduction
        : public gti::ModuleBase<DWaitStateCollReduction, I_DWaitStateCollReduction>, public I_CollCommListener
    {
    public:
        /**
         * Constructor.
         * @param instanceName name of this module instance.
         */
        DWaitStateCollReduction (const char* instanceName);

            /**
             * Destructor.
             */
        virtual ~DWaitStateCollReduction (void);

        /**
         * @see I_DWaitStateCollReduction::request
         */
        GTI_ANALYSIS_RETURN request (
                int isIntercomm,
                unsigned long long contextId,
                int localGroupSize,
                int remoteGroupSize,
                int numTasks,
                    I_ChannelId *cId,
                    std::list<I_ChannelId*> *outFinishedChannels);

        /**
         * @see I_CollCommListener::newCommInColl
         */
        void newCommInColl (
                MustParallelId pId,
                I_CommPersistent* comm);

        /**
         * The timeout function, see gti::I_Reduction::timeout
         */
        void timeout (void);

    protected:
        generateCollectiveActiveRequestP myFForward;
        I_ParallelIdAnalysis* myPIdMod;
        I_CommTrack *myCommTrack;
        I_DCollectiveMatchReduction *myCollMatch;

        struct CommInfo
        {
            int isIntercomm;
            unsigned long long contextId;
            int localSize;
            int remoteSize;
            int numConnected;
            int curRequestNumArrived;
        };

        std::list<CommInfo> myInfos;
        std::list<CommInfo> myUnexpectedRequests;
        bool myInUnexepctedTest;

        /**
         * Returns true if matching information for two communicators is equal.
         * Mimics I_Comm::compareComms implementation
         */
        bool compare (
                int isIntercomm,
                unsigned long long contextId,
                int localGroupSize,
                int remoteGroupSize,
                int isIntercomm2,
                unsigned long long contextId2,
                int localGroupSize2,
                int remoteGroupSize2);
    };
} /*namespace MUST*/

#endif /*DWAITSTATECOLLREDUCTION_H*/
