/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file MsgLoggerDdt.cpp
 *       @see MUST::MsgLoggerDdt.
 *
 *  @date 20.01.2011
 *  @author Tobias Hilbrich
 */

#include "GtiMacros.h"

#include "MsgLoggerDdt.h"

using namespace must;

mGET_INSTANCE_FUNCTION(MsgLoggerDdt);
mFREE_INSTANCE_FUNCTION(MsgLoggerDdt);
mPNMPI_REGISTRATIONPOINT_FUNCTION(MsgLoggerDdt);

//=============================
// Constructor
//=============================
MsgLoggerDdt::MsgLoggerDdt (const char* instanceName)
    : gti::ModuleBase<MsgLoggerDdt, I_MessageLogger> (instanceName),
      myPIdModule (NULL),
      myLIdModule (NULL)
{
    //create sub modules
    std::vector<I_Module*> subModInstances;
    subModInstances = createSubModuleInstances ();

    //save sub modules
    myLIdModule = (I_LocationAnalysis*) subModInstances[0];
    myPIdModule = (I_ParallelIdAnalysis*) subModInstances[1];
}

//=============================
// Destructor
//=============================
MsgLoggerDdt::~MsgLoggerDdt (void)
{
  if (myLIdModule)
    destroySubModuleInstance ((I_Module*) myLIdModule);
  myLIdModule = NULL;

  if (myPIdModule)
    destroySubModuleInstance ((I_Module*) myPIdModule);
  myPIdModule = NULL;
}

//=============================
// log
//=============================
GTI_ANALYSIS_RETURN MsgLoggerDdt::log (
          int msgId,
          int hasLocation,
          uint64_t pId,
          uint64_t lId,
          int msgType,
          char *text,
          int textLen,
          int numReferences,
          uint64_t* refPIds,
          uint64_t* refLIds)
{
    if (!hasLocation)
        return logStrided (msgId, pId, lId, 0, 0, 0, msgType, text, textLen, numReferences, refPIds, refLIds);

    return logStrided (msgId, pId, lId, myPIdModule->getInfoForId(pId).rank, 1, 1, msgType, text, textLen, numReferences, refPIds, refLIds);
}

//=============================
// logStrided
//=============================
GTI_ANALYSIS_RETURN MsgLoggerDdt::logStrided (
              int msgId,
              uint64_t pId,
              uint64_t lId,
              int startRank,
              int stride,
              int count,
              int msgType,
              char *text,
              int textLen,
              int numReferences,
              uint64_t* refPIds,
              uint64_t* refLIds
      )
{
    switch ((MustMessageType)msgType)
    {
    case MustErrorMessage:
        break;
    default:
        return GTI_ANALYSIS_SUCCESS;
    }

    myDdtBreakpointFunction(text);

    return GTI_ANALYSIS_SUCCESS;
}

//=============================
// myDdtBreakpointFunction
//=============================
void myDdtBreakpointFunction(char* text)
{
  
}

/*EOF*/
