/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file MsgLoggerDdt.h
 *       @see MUST::MsgLoggerDdt.
 *
 *  @date 11.10.2012
 *  @author Joachim Protze
 */

#include "ModuleBase.h"
#include "I_MessageLogger.h"
#include "I_ParallelIdAnalysis.h"
#include "I_LocationAnalysis.h"

#ifndef MSGLOGGERDDT_H
#define MSGLOGGERDDT_H

using namespace gti;

void myDdtBreakpointFunction(char* text);

namespace must
{
  /**
     * Implementation of I_MessageLogger that prints all logged events to
     * std::cout.
     */
    class MsgLoggerDdt : public gti::ModuleBase<MsgLoggerDdt, I_MessageLogger>
    {
    public:
      /**
       * Constructor.
       * @param instanceName name of this module instance.
       */
      MsgLoggerDdt (const char* instanceName);

      /**
       * Destructor.
       */
      virtual ~MsgLoggerDdt (void);

      /**
       * @see I_MessageLogger::log.
       */
      GTI_ANALYSIS_RETURN log (
          int msgId,
          int hasLocation,
          uint64_t pId,
          uint64_t lId,
          int msgType,
          char *text,
          int textLen,
          int numReferences,
          uint64_t* refPIds,
          uint64_t* refLIds
      );

      /**
       * @see I_MessageLogger::logStrided.
       */
      GTI_ANALYSIS_RETURN logStrided (
              int msgId,
              uint64_t pId,
              uint64_t lId,
              int startRank,
              int stride,
              int count,
              int msgType,
              char *text,
              int textLen,
              int numReferences,
              uint64_t* refPIds,
              uint64_t* refLIds
      );

    protected:
      I_ParallelIdAnalysis *myPIdModule;
      I_LocationAnalysis *myLIdModule;
    }; /*class MsgLoggerDdt */
} /*namespace MUST*/

#endif /*MSGLOGGERDDT_H*/
