/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file DatatypePredefs.cpp
 *       @see MUST::DatatypePredefs.
 *
 *  @date 18.02.2011
 *  @author Tobias Hilbrich
 */

#include "mustFeaturetested.h"
#include "mustConfig.h"

#include "GtiMacros.h"

#include "DatatypePredefs.h"
#include "I_DatatypeTrack.h"
#include "ResourceApi.h"

#include <mpi.h>

using namespace must;

mGET_INSTANCE_FUNCTION(DatatypePredefs);
mFREE_INSTANCE_FUNCTION(DatatypePredefs);
mPNMPI_REGISTRATIONPOINT_FUNCTION(DatatypePredefs);

/*
 * Prototypes for Fortran helper functions that are
 * used to return the handle values for these types.
 */
extern "C" void get_mpi_integer_code___ (int* code);
extern "C" void get_mpi_real_code___ (int* code);
extern "C" void get_mpi_double_precision_code___ (int* code);
extern "C" void get_mpi_complex_code___ (int* code);
extern "C" void get_mpi_logical_code___ (int* code);
extern "C" void get_mpi_character_code___ (int* code);

extern "C" void get_mpi_2real_code___ (int* code);
extern "C" void get_mpi_2double_precision_code___ (int* code);
extern "C" void get_mpi_2integer_code___ (int* code);
extern "C" void get_mpi_2complex_code___ (int* code);
extern "C" void get_mpi_2double_complex_code___ (int* code);

extern "C" void get_mpi_integer1_code___ (int* code);
extern "C" void get_mpi_integer2_code___ (int* code);
extern "C" void get_mpi_integer4_code___ (int* code);
extern "C" void get_mpi_integer8_code___ (int* code);
extern "C" void get_mpi_integer16_code___ (int* code);
extern "C" void get_mpi_real2_code___ (int* code);
extern "C" void get_mpi_real4_code___ (int* code);
extern "C" void get_mpi_real8_code___ (int* code);
extern "C" void get_mpi_real16_code___ (int* code);
extern "C" void get_mpi_double_complex_code___ (int* code);
extern "C" void get_mpi_complex8_code___ (int* code);
extern "C" void get_mpi_complex16_code___ (int* code);
extern "C" void get_mpi_complex32_code___ (int* code);
extern "C" void get_mpi_logical1_code___ (int* code);
extern "C" void get_mpi_logical2_code___ (int* code);
extern "C" void get_mpi_logical4_code___ (int* code);
extern "C" void get_mpi_logical8_code___ (int* code);
extern "C" void get_mpi_logical16_code___ (int* code);

//=============================
// Constructor
//=============================
DatatypePredefs::DatatypePredefs (const char* instanceName)
    : gti::ModuleBase<DatatypePredefs, I_DatatypePredefs> (instanceName)
{
    //create sub modules
    std::vector<I_Module*> subModInstances;
    subModInstances = createSubModuleInstances ();

    //handle sub modules
    ////no sub modules needed
}

//=============================
// Destructor
//=============================
DatatypePredefs::~DatatypePredefs ()
{
    //Nothing to do
}

#define iNSERTcpREDEF(mustId,mpiId) \
	if (mpiId != MPI_DATATYPE_NULL){predefs.insert(std::make_pair(mustId, MUST_Type_m2i(mpiId))); \
	retValue = PMPI_Type_extent(mpiId, &extent); \
	if (retValue != MPI_SUCCESS) std::cerr << "WARNING: MPI_Type_extent(" << mpiId << ") failed" << std::endl; \
	extents.insert(std::make_pair(mustId, extent)); \
	types[0] = mpiId; \
	displs[1] = extents[mustId]; \
	PMPI_Type_struct (2, blocklens, displs, types, &newType); \
	retValue = PMPI_Type_extent(newType, &extent); \
    if (retValue != MPI_SUCCESS) std::cerr << "WARNING: MPI_Type_extent(" << mpiId << ") failed" << std::endl; \
	alignments.insert(std::make_pair(mustId, extent - extents[mustId])); \
    PMPI_Type_free(&newType);}

#define iNSERTfORTRANpREDEF(mustId,mpiId) \
    types[0] = MUST_Type_f2c(mpiId); \
	if (mpiId != MUST_Type_c2f(MPI_DATATYPE_NULL)){predefs.insert(std::make_pair(mustId, MUST_Type_m2i(types[0]))); \
	retValue = PMPI_Type_extent(types[0], &extent); \
    if (retValue != MPI_SUCCESS) std::cerr << "WARNING: MPI_Type_extent(" << mpiId << ") failed" << std::endl; \
	extents.insert(std::make_pair(mustId, extent)); \
	displs[1] = extents[mustId]; \
	PMPI_Type_struct (2, blocklens, displs, types, &newType); \
	retValue = PMPI_Type_extent(newType, &extent); \
    if (retValue != MPI_SUCCESS) std::cerr << "WARNING: MPI_Type_extent(" << mpiId << ") failed" << std::endl; \
	alignments.insert(std::make_pair(mustId, extent - extents[mustId])); \
    PMPI_Type_free(&newType);}


//=============================
// propagate
//=============================
GTI_ANALYSIS_RETURN DatatypePredefs::propagate (
                MustParallelId pId
)
{
	std::map<MustMpiDatatypePredefined, MustDatatypeType> predefs;
	std::map<MustMpiDatatypePredefined, int> alignments;
	std::map<MustMpiDatatypePredefined, MPI_Aint> extents;
	int handle;
    int retValue;
	int fortranNull = MUST_Type_c2f (MPI_DATATYPE_NULL);

	//For geting extents
	MPI_Aint extent;

	//For getting alignments
	MPI_Datatype types[2] = {MPI_DATATYPE_NULL, MPI_CHAR}; //first value needs to be adapted
	int blocklens[2] = {1,1};
	MPI_Aint displs[2] = {0,0}; //second value needs to be adapted
	MPI_Datatype newType;

	//==Add all the predefined values to the map
	//=1) Elementary C
	iNSERTcpREDEF(MUST_MPI_CHAR, MPI_CHAR);
	iNSERTcpREDEF(MUST_MPI_SHORT, MPI_SHORT);
	iNSERTcpREDEF(MUST_MPI_INT, MPI_INT);
	iNSERTcpREDEF(MUST_MPI_LONG, MPI_LONG);
	iNSERTcpREDEF(MUST_MPI_UNSIGNED_CHAR, MPI_UNSIGNED_CHAR);
	iNSERTcpREDEF(MUST_MPI_UNSIGNED_SHORT, MPI_UNSIGNED_SHORT);
	iNSERTcpREDEF(MUST_MPI_UNSIGNED, MPI_UNSIGNED);
	iNSERTcpREDEF(MUST_MPI_UNSIGNED_LONG, MPI_UNSIGNED_LONG);
	iNSERTcpREDEF(MUST_MPI_FLOAT, MPI_FLOAT);
	iNSERTcpREDEF(MUST_MPI_DOUBLE, MPI_DOUBLE);
	iNSERTcpREDEF(MUST_MPI_LONG_DOUBLE, MPI_LONG_DOUBLE);

	//=2) Elementary C & Fortran
	iNSERTcpREDEF(MUST_MPI_BYTE, MPI_BYTE);
	iNSERTcpREDEF(MUST_MPI_PACKED, MPI_PACKED);

	//=3) Elementary Fortran
#ifdef GTI_ENABLE_FORTRAN
	get_mpi_integer_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER, handle);
	}
	get_mpi_real_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_REAL, handle);
	}
	get_mpi_double_precision_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_DOUBLE_PRECISION, handle);
	}
	get_mpi_complex_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_COMPLEX, handle);
	}
	get_mpi_logical_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL, handle);
	}
	get_mpi_character_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_CHARACTER, handle);
	}
#endif

	//=4) Reduction types C
	iNSERTcpREDEF(MUST_MPI_FLOAT_INT, MPI_FLOAT_INT);
	iNSERTcpREDEF(MUST_MPI_DOUBLE_INT, MPI_DOUBLE_INT);
	iNSERTcpREDEF(MUST_MPI_LONG_INT, MPI_LONG_INT);
	iNSERTcpREDEF(MUST_MPI_2INT, MPI_2INT);
	iNSERTcpREDEF(MUST_MPI_SHORT_INT, MPI_SHORT_INT);
	iNSERTcpREDEF(MUST_MPI_LONG_DOUBLE_INT, MPI_LONG_DOUBLE_INT);

	//=5) Reduction types Fortran
#ifdef GTI_ENABLE_FORTRAN
	get_mpi_2real_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_2REAL, handle);
	}
	get_mpi_2double_precision_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_2DOUBLE_PRECISION, handle);
	}
	get_mpi_2integer_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_2INTEGER, handle);
	}
	get_mpi_2complex_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_2COMPLEX, handle);
	}
	get_mpi_2double_complex_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_2DOUBLE_COMPLEX, handle);
	}
#endif

	//=6) Optional C
#ifdef HAVE_MPI_LONG_LONG_INT
	iNSERTcpREDEF(MUST_MPI_LONG_LONG_INT, MPI_LONG_LONG_INT);
#endif
#ifdef HAVE_MPI_LONG_LONG
	iNSERTcpREDEF(MUST_MPI_LONG_LONG, MPI_LONG_LONG);
#endif
#ifdef HAVE_MPI_UNSIGNED_LONG_LONG
	iNSERTcpREDEF(MUST_MPI_UNSIGNED_LONG_LONG, MPI_UNSIGNED_LONG_LONG);
#endif
#ifdef HAVE_MPI_WCHAR
	iNSERTcpREDEF(MUST_MPI_WCHAR, MPI_WCHAR);
#endif
#ifdef HAVE_MPI_SIGNED_CHAR
	iNSERTcpREDEF(MUST_MPI_SIGNED_CHAR, MPI_SIGNED_CHAR);
#endif

	//=7) Optional Fortran
#ifdef GTI_ENABLE_FORTRAN
	get_mpi_integer1_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER1, handle);
	}
	get_mpi_integer2_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER2, handle);
	}
	get_mpi_integer4_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER4, handle);
	}
	get_mpi_integer8_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER8, handle);
	}
	get_mpi_integer16_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_INTEGER16, handle);
	}
	get_mpi_real2_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_REAL2, handle);
	}
	get_mpi_real4_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_REAL4, handle);
	}
	get_mpi_real8_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_REAL8, handle);
	}
	get_mpi_real16_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_REAL16, handle);
	}
	get_mpi_double_complex_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_DOUBLE_COMPLEX, handle);
	}
	get_mpi_complex8_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_COMPLEX8, handle);
	}
	get_mpi_complex16_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_COMPLEX16, handle);
	}
	get_mpi_complex32_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_COMPLEX32, handle);
	}
	get_mpi_logical1_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL1, handle);
	}
	get_mpi_logical2_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL2, handle);
	}
	get_mpi_logical4_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL4, handle);
	}
	get_mpi_logical8_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL8, handle);
	}
	get_mpi_logical16_code___ (&handle);
	if (handle != fortranNull)
	{
		iNSERTfORTRANpREDEF(MUST_MPI_LOGICAL16, handle);
	}
#endif

	//=8) Bound markers
	iNSERTcpREDEF(MUST_MPI_UB, MPI_UB);
    alignments[MUST_MPI_UB]=1;
	iNSERTcpREDEF(MUST_MPI_LB, MPI_LB);
    alignments[MUST_MPI_LB]=1;

	//==Call the API call to propagate the message to the I_DatatypeTrack
	propagatePredefinedDatatypesP fP;
	if (getWrapperFunction ("propagatePredefinedDatatypes", (GTI_Fct_t*)&fP) == GTI_SUCCESS)
	{
		int* enumValues = new int [predefs.size()];
		MustDatatypeType* handleValues = new MustDatatypeType [predefs.size()];
		int* alignmentValues = new int [predefs.size()];
		MustAddressType* extentValues = new MustAddressType [predefs.size()];

		std::map<MustMpiDatatypePredefined, MustDatatypeType>::iterator iter;
		int i = 0;
		for (iter = predefs.begin(); iter != predefs.end(); iter++, i++)
		{
			enumValues[i] = (int)iter->first;
			handleValues[i] = iter->second;

			//Extent and Alignment
			assert (alignments.find (iter->first) != alignments.end()); //Internal error, did not calculate an alignment
			assert (extents.find (iter->first) != extents.end()); //Internal error, did not calculate an extent
			alignmentValues[i] = alignments[iter->first];
			extentValues[i] = (MustAddressType)extents[iter->first];
		}

		(*fP) (
            pId,
			MUST_Type_m2i (MPI_DATATYPE_NULL),
			predefs.size(),
			enumValues,
			handleValues,
			extentValues,
			alignmentValues
			);

		if (enumValues)
		    delete[] enumValues;
		if (handleValues)
		    delete[] handleValues;
		if (alignmentValues)
		    delete[] alignmentValues;
		if (extentValues)
		    delete[] extentValues;
	}
	else
	{
		////std::cout << "ERROR: failed to get \"propagatePredefinedDatatypes\" function pointer from wrapper. Did you forgot to load the API group MUST_Resource_API ? Aborting now." << std::endl;
		////return GTI_ANALYSIS_FAILURE;
		//We fail silently here, the function won't exist if no datatype tracker is mapped ....
	}

	return GTI_ANALYSIS_SUCCESS;
}

/*EOF*/
